local PounceCam = LibStub("AceAddon-3.0"):NewAddon("PounceCam", "AceEvent-3.0", "AceTimer-3.0")

--==== upvalues ====--
local _G, _, LibStub, format, ipairs, select = _G, _, LibStub, format, ipairs, select
local GetCVar, SetCVar, GetSpellInfo, UnitClass, UnitGUID = GetCVar, SetCVar, GetSpellInfo, UnitClass, UnitGUID
--==================--


local CEvent = { module = PounceCam }
CEvent.__index = CEvent
function CEvent:Release()
	if self.handler then 
		self.module:UnregisterEvent(self.eventName)
		self.handler = nil
	end
end
function CEvent:Set(handler)
	if handler == self.handler then return end
	self:Release()
	self.module:RegisterEvent(self.eventName, handler)
	self.handler = handler
end
local Event_CLEU = setmetatable( { eventName = "COMBAT_LOG_EVENT_UNFILTERED" }, CEvent )

local function NameWithForm(id)
	local name, rank = GetSpellInfo(id)
	return format("%s (%s)", name, rank)
end

local db
local strPlayerGUID
local strPlayerClass
local strCameraFollowStyle
local strCameraFollowSpeed

local idSpellCatForm = 768
local idSpellBearForm = 5487
-- local idSpellAquaticForm = 1066
-- local idSpellTravelForm = 783
-- local idSpellMoonkinForm = 24858

local idSpellWildCharge = 102401
local idSpellWildChargeCat = 49376
local idSpellWildChargeBear = 16979
-- local idSpellWildChargeTravel = 102417
-- local idSpellWildChargeMoonkin = 102383
-- local idSpellWildChargeAquatic = 102416

local idSpellSkullBash = 106839
local idSpellSkullBashCat = 80965
local idSpellSkullBashBear = 80964

local idSpellWarriorCharge = 100
local idSpellWarriorIntervene = 3411
-- local idSpellWarriorIntercept = 20252 -- removed in MoP 

local tblOptionsDefaults = {
	global = {
		CameraResetDelay            = 1,

		[idSpellWildCharge]         = true,
		[idSpellWildChargeCat]      = true,
		[idSpellWildChargeBear]     = true,

		[idSpellSkullBashCat]       = false,
		[idSpellSkullBashBear]      = false,

		[idSpellWarriorCharge]      = true,
		[idSpellWarriorIntervene]   = true,
--		[idSpellWarriorIntercept]   = true, -- removed in MoP 
	}
}

local tblMain = {
	["DRUID"] = { 
		{ type = "description", name = " " },
		{ type = "description", name = GetSpellInfo(idSpellWildCharge) },
		{ type = "toggle", idSpell = idSpellWildChargeBear, name = GetSpellInfo(idSpellBearForm) },
		{ type = "toggle", idSpell = idSpellWildChargeCat,  name = GetSpellInfo(idSpellCatForm)  },
		{ type = "toggle", idSpell = idSpellWildCharge,     name = "non-shapeshifted" },

		{ type = "description", name = " " },
		{ type = "description", name = GetSpellInfo(idSpellSkullBash) },
		{ type = "toggle", idSpell = idSpellSkullBashBear,  name = GetSpellInfo(idSpellBearForm) },
		{ type = "toggle", idSpell = idSpellSkullBashCat,   name = GetSpellInfo(idSpellCatForm)  },
	},
	["WARRIOR"] = {
		{ type = "description", name = " " },
		{ type = "toggle", idSpell = idSpellWarriorCharge    },
		{ type = "toggle", idSpell = idSpellWarriorIntervene },
	 -- { type = "toggle", idSpell = idSpellWarriorIntercept }, -- removed in MoP 
	}
}

local function CheckHandler(strClass)
	for _,v in ipairs(tblMain[strPlayerClass] or {}) do
		if (v.idSpell and db[v.idSpell]) then
			Event_CLEU:Set("CLEU")
			return
		end
	end

	Event_CLEU:Release()
end


-- Called when the addon is loaded
function PounceCam:OnInitialize()
	db = (LibStub("AceDB-3.0"):New("PounceCamDB", tblOptionsDefaults)).global
end

-- Called when the addon is enabled
function PounceCam:OnEnable()
	_, strPlayerClass = UnitClass("player")
	if tblMain[strPlayerClass] then 
		LibStub("AceConfig-3.0"):RegisterOptionsTable("PounceCam", PounceCam:GetOptionsUI(strPlayerClass))
		self.optionsFrame = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("PounceCam")

		strPlayerGUID = UnitGUID("player")
		CheckHandler()
	end
end

-- Called when the addon is Disabled
function PounceCam:OnDisable()
	Event_CLEU:Release()
end


function PounceCam:ResetCameraFollow()
	SetCVar("cameraSmoothStyle",    strCameraFollowStyle)
	SetCVar("cameraYawSmoothSpeed", strCameraFollowSpeed)
	-- print("resetting camera")
end

function PounceCam:CLEU(_, ...)
	local eventType = select(2, ...)
	local sourceGUID = select(4, ...)

	if(sourceGUID == strPlayerGUID and eventType == "SPELL_CAST_SUCCESS") then
		local spellId, spellName = select(12, ...)
		if (db[spellId]) then
			-- print("pounce cam firing for spell", spellId, spellName)
			strCameraFollowStyle = GetCVar("cameraSmoothStyle")
			strCameraFollowSpeed = GetCVar("cameraYawSmoothSpeed")

			SetCVar("cameraSmoothStyle",    "3")
			SetCVar("cameraYawSmoothSpeed", "270")
			PounceCam:ScheduleTimer("ResetCameraFollow", db.CameraResetDelay)
		end	
	end
end

function PounceCam:GetOptionsUI(strClass)
	local tblArgs = {}
	local index = 1

	tblArgs["PounceCamWidget0"] = {
		order = index,
		type = "range",
		name = "Camera Reset Delay",
		desc = "amount of time to wait before resetting camera follow style back to user preference",
		min = 0,
		softMin = .5,
		softMax = 3,
		get = function(info) return db.CameraResetDelay end,
		set = function(info, value) db.CameraResetDelay = value end,
	}

	for _,v in ipairs(tblMain[strPlayerClass] or {}) do
		index = index + 1
		if (v.type == "toggle") then
			local name, _, icon = GetSpellInfo(v.idSpell)
			tblArgs["PounceCamWidget" .. index] = {
				order = index,
				width = "full",
				type = "toggle",
				name = v.name or name,
				image = v.icon or icon,
				get = function(info) return db[v.idSpell] end,
				set = function(info, value) 
					db[v.idSpell] = value 
					CheckHandler()
				end,
			}
		elseif (v.type == "description") then
			tblArgs["PounceCamWidget" .. index] = {
				order = index,
				width = "full",
				type = "description",
				fontSize = "large",
				name = v.name,
			}
		end
	end

	return {
		name = "PounceCam",
		handler = PounceCam,
		type = 'group',	
		childGroups = "tree",
		args = tblArgs,
		}
end

